@@ -55,10 +55,8 @@ QT_END_NAMESPACE
 # include "qcoreapplication.h"
 #endif
 
-#ifndef QT_BUILD_QMAKE_BOOTSTRAP
-#  include "private/qglobal_p.h"
-#  include "qconfig.cpp"
-#endif
+#include "private/qglobal_p.h"
+#include "qconfig.cpp"
 
 #ifdef Q_OS_DARWIN
 #  include "private/qcore_mac_p.h"
@@ -184,7 +182,7 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
 {
 #ifdef QT_BUILD_QMAKE
     QString qtconfig = qmake_libraryInfoFile();
-    if (QFile::exists(qtconfig))
+    if (!qtconfig.isEmpty())
         return new QSettings(qtconfig, QSettings::IniFormat);
 #else
     QString qtconfig = QStringLiteral(":/qt/etc/qt.conf");
@@ -207,6 +205,9 @@ QSettings *QLibraryInfoPrivate::findConfiguration()
 #endif
     if (QCoreApplication::instance()) {
         QDir pwd(QCoreApplication::applicationDirPath());
+        qtconfig = pwd.filePath(QLatin1String("qt" QT_STRINGIFY(QT_VERSION_MAJOR) ".conf"));
+        if (QFile::exists(qtconfig))
+            return new QSettings(qtconfig, QSettings::IniFormat);
         qtconfig = pwd.filePath(QLatin1String("qt.conf"));
         if (QFile::exists(qtconfig))
             return new QSettings(qtconfig, QSettings::IniFormat);
@@ -292,8 +293,10 @@ QLibraryInfo::QLibraryInfo()
 #    define COMPILER_STRING "MSVC 2015"
 #  elif _MSC_VER < 1917
 #    define COMPILER_STRING "MSVC 2017"
-#  elif _MSC_VER < 2000
+#  elif _MSC_VER < 1930
 #    define COMPILER_STRING "MSVC 2019"
+#  elif _MSC_VER < 2000
+#    define COMPILER_STRING "MSVC 2022"
 #  else
 #    define COMPILER_STRING "MSVC _MSC_VER " QT_STRINGIFY(_MSC_VER)
 #  endif
@@ -364,8 +367,14 @@ QVersionNumber QLibraryInfo::version() noexcept
  * - add to qt_config.prf, qt_module.prf, qt_module_fwdpri.prf
  */
 
+#ifdef QT_BUILD_QMAKE
+constexpr size_t qtConfEntriesKeySize = 23;
+#else
+constexpr size_t qtConfEntriesKeySize = 19;
+#endif
+
 static const struct {
-    char key[19], value[13];
+    char key[qtConfEntriesKeySize], value[13];
 } qtConfEntries[] = {
     { "Prefix", "." },
     { "Documentation", "doc" }, // should be ${Data}/doc
@@ -388,6 +397,11 @@ static const struct {
     { "Sysroot", "" },
     { "SysrootifyPrefix", "" },
     { "HostBinaries", "bin" },
+#ifdef Q_OS_WIN
+    { "HostLibraryExecutables", "bin" },
+#else
+    { "HostLibraryExecutables", "libexec" },
+#endif
     { "HostLibraries", "lib" },
     { "HostData", "." },
     { "TargetSpec", "" },
@@ -592,7 +606,7 @@ static QString getRelocatablePrefix()
 }
 #endif
 
-#if defined(QT_BUILD_QMAKE) && !defined(QT_BUILD_QMAKE_BOOTSTRAP)
+#if defined(QT_BUILD_QMAKE)
 QString qmake_abslocation();
 
 static QString getPrefixFromHostBinDir(const char *hostBinDirToPrefixPath)
@@ -613,7 +627,6 @@ static QString getHostPrefixFromHostBinDir()
 }
 #endif
 
-#ifndef QT_BUILD_QMAKE_BOOTSTRAP
 static QString getPrefix(
 #ifdef QT_BUILD_QMAKE
         QLibraryInfo::PathGroup group
@@ -634,7 +647,6 @@ static QString getPrefix(
     return QString::fromLocal8Bit(QT_CONFIGURE_PREFIX_PATH);
 #endif
 }
-#endif // QT_BUILD_QMAKE_BOOTSTRAP
 
 /*! \fn QString QLibraryInfo::location(LibraryLocation loc)
     \obsolete Use path() instead.
@@ -751,7 +763,6 @@ QLibraryInfo::rawLocation(LibraryPath loc, PathGroup group)
     }
 #endif // settings
 
-#ifndef QT_BUILD_QMAKE_BOOTSTRAP
     if (!fromConf) {
         // "volatile" here is a hack to prevent compilers from doing a
         // compile-time strlen() on "path". The issue is that Qt installers
@@ -781,7 +792,6 @@ QLibraryInfo::rawLocation(LibraryPath loc, PathGroup group)
         if (path)
             ret = QString::fromLocal8Bit(path);
     }
-#endif
 
 #ifdef QT_BUILD_QMAKE
     // These values aren't actually paths and thus need to be returned verbatim.
